// ########################
// ### Global Variables ###
// ########################
array<string> months = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
uint monthId;
uint year;
int Setting_CurrentMonth = 0;

void Main()
{
    string ToTDsResponse = SendJSONRequest(Net::HttpMethod::Get, "https://trackmania.io/api/totd/" + Setting_CurrentMonth);
    Json::Value ToTDs = ResponseToJSON(ToTDsResponse, Json::Type::Object);
    monthId = ToTDs["month"] - 1;
    year = ToTDs["year"];
}

// ################
// ### Settings ###
// ################

[Setting name="Show Main Window" hidden]
bool Setting_WindowShown = false;

[Setting name="Non-Gold Medal only" description="Play the maps where you don't have gold medal yet"]
bool Setting_GoldMedalOnly = false;

[Setting name="Non-Author Medal only" description="Play the maps where you don't have Author medal yet"]
bool Setting_AuthorMedalOnly = false;



// #######################################################
// ### Network Requests (imported from AFK Queue Tool) ###
// #######################################################

string SendNadeoRequest(const Net::HttpMethod Method, const string &in URL, string Body = "") {
    dictionary@ Headers = dictionary();
    auto app = cast<CTrackMania>(GetApp());
    Headers["Accept"] = "application/json";
    Headers["Content-Type"] = "application/json";
    Headers["Authorization"] = "nadeo_v1 t=" + app.ManiaPlanetScriptAPI.Authentication_Token;
    return SendHTTPRequest(Method, URL, Body, Headers);
}

string SendJSONRequest(const Net::HttpMethod Method, const string &in URL, string Body = "") {
    dictionary@ Headers = dictionary();
    Headers["Accept"] = "application/json";
    Headers["Content-Type"] = "application/json";
    return SendHTTPRequest(Method, URL, Body, Headers);
}

string SendHTTPRequest(const Net::HttpMethod Method, const string &in URL, const string &in Body, dictionary@ Headers) {
    Net::HttpRequest req;
    req.Method = Method;
    req.Url = URL;
    @req.Headers = Headers;
    req.Body = Body;
    req.Start();
    while (!req.Finished()) {
        yield();
    }
    return req.String();
}

Json::Value ResponseToJSON(const string &in HTTPResponse, Json::Type ExpectedType) {
    Json::Value ReturnedObject;
    try {
        ReturnedObject = Json::Parse(HTTPResponse);
    } catch {
        print("JSON Parsing of string failed!");
        //Error(ErrorType::Error, "ResponseToJSON", "JSON Parsing of string failed!", HTTPResponse);
    }
    if (ReturnedObject.GetType() != ExpectedType) {
        print("Unexpected JSON Type returned");
        //Error(ErrorType::Warn, "ResponseToJSON", "Unexpected JSON Type returned", HTTPResponse);
        return ReturnedObject;
    }
    return ReturnedObject;
}

// #################################
// ### Render Menu and Interface ###
// #################################


void RenderMenu()
{
    if (UI::MenuItem("ToTD Random Picker")) {
        Setting_WindowShown = !Setting_WindowShown;
    }
}

void RenderInterface()
{
    if (!Setting_WindowShown) return;
    if (UI::Begin("ToTD Random Picker: " + months[monthId] + " " + year, Setting_WindowShown, UI::WindowFlags::NoCollapse)) {

        if (months[Math::Abs((monthId - 1)%12)] + " " + year != "June 2020"){

            string monthBefore;
            
            if (monthId == 0){ monthBefore = "December" + (year - 1); } else { monthBefore = months[monthId - 1] + " " + year; }

            if (UI::Button(monthBefore)){
                Setting_CurrentMonth = Setting_CurrentMonth + 1;
                if (monthId - 1 == -1){
                    monthId = 11;
                    year = year - 1;
                } else {
                    monthId = monthId - 1;
                }
            }
        }

        if (Setting_CurrentMonth > 0){
            UI::SameLine();

            if (UI::Button(months[(monthId + 1)%12] + " " + (year + (1*(monthId + 1)/12)))){
                Setting_CurrentMonth = Setting_CurrentMonth - 1;
                if (monthId + 1 == 12){
                    monthId = 0;
                    year = year + 1;
                } else {
                    monthId = monthId + 1;
                }
            }
        }

        if((Permissions::PlayCurrentOfficialMonthlyCampaign() && Setting_CurrentMonth == 0) || (Permissions::PlayPastOfficialMonthlyCampaign() && Setting_CurrentMonth != 0)){
            if (UI::Button("Play a Random ToTD map")){
                startnew(PlayRandomToTD);
            }
        }

        UI::End();  
    }
}

void PlayRandomToTD(){

    auto app = cast<CTrackMania>(GetApp());
    
    string ToTDsResponse = SendJSONRequest(Net::HttpMethod::Get, "https://trackmania.io/api/totd/" + Setting_CurrentMonth);
    Json::Value ToTDs = ResponseToJSON(ToTDsResponse, Json::Type::Object);

    Json::Value[] maps = {};

    for (uint i = ToTDs["days"].Length; i > 0; i--) {
        maps.InsertLast(ToTDs["days"][i-1]);
    }

    uint mapId = Math::Rand(0, maps.Length - 1);

    /*
    if (Setting_GoldMedalOnly || Setting_AuthorMedalOnly) {
        string MapRecordResponse = SendNadeoRequest(Net::HttpMethod::Get, "/mapRecords/?accountIdList=" + )
    }*/

    string fileUrl = maps[mapId]["map"]["fileUrl"];
    
    app.ManiaTitleControlScriptAPI.PlayMap(fileUrl, "", "");
}